import numpy
from nltk.corpus import brown
import nltk

genres = ['news', 'religion', 'hobbies', 'science_fiction', 'romance', 'humor']


#brown_sents_label_pre=[[ [sent,g] for sent in nltk.corpus.brown.sents(categories=[g]) ] for g in genres]
cfd = nltk.ConditionalFreqDist( (genre, word) for genre in brown.categories()  for word in brown.words(categories=genre))
genres = ['news', 'religion', 'hobbies', 'science_fiction', 'romance', 'humor']
modals = ['can', 'could', 'may', 'might', 'must', 'will']
cfd.tabulate(conditions=genres, samples=modals)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Licensed under the GNU Affero General Public License, version 3 - http://www.gnu.org/licenses/agpl-3.0.html

import math
import copy
from Queue import Queue

from numpy import zeros, random, sum as np_sum, add as np_add, concatenate, \
    repeat as np_repeat, array, float32 as REAL, empty, ones, memmap as np_memmap, \
    sqrt, newaxis, ndarray, dot, vstack, dtype, divide as np_divide

import gensim.models.word2vec
import gensim.models.doc2vec 

from six.moves import xrange, zip
from six import string_types, integer_types, itervalues

import sys
import random

import numpy as np
import operator

import keras.constraints

from keras.utils.np_utils import accuracy
from keras.models import Graph,Sequential
from keras.layers.core import Dense, Dropout, Activation, Merge, Flatten, LambdaMerge,Lambda
from keras.layers.embeddings import Embedding
from keras.optimizers import SGD
from keras.objectives import mse

from word2veckeras import train_sg_pair,train_cbow_pair,queue_to_list


def train_batch_dbow(model,
                     docs, alpha,
                     work=None,
                     train_words=False, learn_doctags=True, learn_words=True, learn_hidden=True,
                     word_vectors=None, word_locks=None, doctag_vectors=None, doctag_locks=None,
                     batch_size=100):
    #print 'train_batch_dbow'
    batch_count=0
    train_x0=[[0]]*batch_size
    train_x1=[[0]]*batch_size
    train_y=[[0]]*batch_size
    while 1:
        for doc in docs:
            for doctag_index in doc.tags:
                for word in doc.words:
                    xy=train_sg_pair(model, word, doctag_index, alpha, learn_vectors=learn_doctags,
                              learn_hidden=learn_hidden, context_vectors=doctag_vectors,
                              context_locks=doctag_locks)
                    if xy !=None:
                        (x0,x1,y)=xy
                        #print xy
                        train_x0[batch_count]=[x0]
                        train_x1[batch_count]=x1
                        train_y[batch_count]=y
                        batch_count += 1
                        if batch_count >= batch_size :
                            yield { 'index':np.array(train_x0), 'point':np.array(train_x1), 'code':np.array(train_y)}
                            batch_count=0

                            
def train_batch_dm_xy_generator(model, docs):
    for doc in docs:
        indexed_doctags = model.docvecs.indexed_doctags(doc.tags)
        doctag_indexes, doctag_vectors, doctag_locks, ignored = indexed_doctags

        word_vocabs = [model.vocab[w] for w in doc.words if w in model.vocab and
                           model.vocab[w].sample_int > model.random.rand() * 2**32]
        for pos, word in enumerate(word_vocabs):
            reduced_window = model.random.randint(model.window)  # `b` in the original doc2vec code
            start = max(0, pos - model.window + reduced_window)
            window_pos = enumerate(word_vocabs[start:(pos + model.window + 1 - reduced_window)], start)
            word2_indexes = [word2.index for pos2, word2 in window_pos if pos2 != pos]
                
            xy=train_cbow_pair(model, word, word2_indexes) #, l1=None, alpha=None,learn_vectors=False, learn_hidden=learn_hidden)
            x2=doctag_indexes
            yield [xy[0],x2,xy[1],xy[2]]
            
                           
def train_batch_dm(model, docs,batch_size=100):
    w_len_queue_dict={}
    w_len_queue=[]
    while 1:
       for xy in train_batch_dm_xy_generator(model,docs):
            if xy != None:
                #print xy
                w_len=len(xy[0])
                if w_len>0:
                    if w_len not in w_len_queue_dict:
                        w_len_queue_dict[w_len]=Queue()
                        w_len_queue.append(w_len)
                    w_len_queue_dict[w_len].put(xy)
            for w_len in w_len_queue:
                #print w_len,w_len_queue_dict[w_len]._qsize()
                if w_len_queue_dict[w_len].qsize() >= batch_size :
                    #print w_len_queue_dict[w_len]
                    l=queue_to_list(w_len_queue_dict[w_len],batch_size)
                    #train=zip(l)
                    #print [w_len,len(l),[[wl,w_len_queue_dict[wl].qsize()] for wl in w_len_queue ]]
                    train=[[e[i] for e in l] for i in range(4)]
                    yield {'iword':np.array(train[0]),
                           'index':np.array(train[1]),
                            'point':np.array(train[2]),
                            'code':np.array(train[3])}
                    
def train_document_dm_concat_xy_generator(model, docs):
    for doc in docs:
        indexed_doctags = model.docvecs.indexed_doctags(doc.tags)
        doctag_indexes, doctag_vectors, doctag_locks, ignored = indexed_doctags

        word_vocabs = [model.vocab[w] for w in doc.words if w in model.vocab and
                       model.vocab[w].sample_int > model.random.rand() * 2**32]
        #doctag_len = len(doctag_indexes)
        #if doctag_len != model.dm_tag_count:

        null_word = model.vocab['\0']
        pre_pad_count = model.window
        post_pad_count = model.window
        padded_document_indexes = (
            (pre_pad_count * [null_word.index])  # pre-padding
            + [word.index for word in word_vocabs if word is not None]  # elide out-of-Vocabulary words
            + (post_pad_count * [null_word.index])  # post-padding
        )

        for pos in range(pre_pad_count, len(padded_document_indexes) - post_pad_count):
            word_context_indexes = (
                padded_document_indexes[(pos - pre_pad_count): pos]  # preceding words
                + padded_document_indexes[(pos + 1):(pos + 1 + post_pad_count)]  # following words
            )
            #word_context_len = len(word_context_indexes)
            #print word_context_len
            predict_word = model.vocab[model.index2word[padded_document_indexes[pos]]]
            xy = train_cbow_pair(model, predict_word, word_context_indexes)
            x2=doctag_indexes
            xy1=[xy[0],x2,xy[1],xy[2]]
            yield xy1
 
def train_document_dm_concat(model, docs,batch_size=100):
    batch_count=0
    train_t=[0]*batch_size

    while 1:
        for xy in train_document_dm_concat_xy_generator(model,docs):
            train_t[batch_count]=xy
            batch_count += 1
            if batch_count >= batch_size :
                train=[[train_t[j][i] for j in range(batch_size) ] for i in range(4)]
                yield {'iword':np.array(train[0]),
                           'index':np.array(train[1]),
                            'point':np.array(train[2]),
                            'code':np.array(train[3])}
                batch_count=0
                      

def build_keras_model_dbow(index_size,vector_size,vocab_size,code_dim,learn_doctags=True, learn_words=True, learn_hidden=True,model=None):
    kerasmodel = Graph()
    kerasmodel.add_input(name='point', input_shape=(code_dim,), dtype=REAL)
    kerasmodel.add_node(kerasmodel.inputs['point'],name='pointnode')
    
    kerasmodel.add_input(name='index' , input_shape=(1,), dtype=int)

    kerasmodel.add_node(Embedding(index_size, vector_size, input_length=1),name='embedding', input='index')
    kerasmodel.add_node(Flatten(),name='embedflatten',input='embedding')
    kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),weights=[model.syn1.T,np.zeros((code_dim))]), name='sigmoid', input='embedflatten')
    kerasmodel.add_output(name='code',inputs=['sigmoid','pointnode'], merge_mode='mul')
    kerasmodel.compile('rmsprop', {'code':'mse'})

    return kerasmodel


def build_keras_model_dm(index_size,vector_size,vocab_size,code_dim,
                         cbow_mean=False,
                         learn_doctags=True, learn_words=True, learn_hidden=True,
                         model=None ,
                         word_vectors=None,doctag_vectors=None,hidden_vectors=None
                         ):
 
    kerasmodel = Graph()

    kerasmodel.add_input(name='iword' , input_shape=(1,), dtype=int)
    kerasmodel.add_input(name='index' , input_shape=(1,), dtype=int)
    
    # embedword=Embedding(vocab_size, vector_size,
    #                     #trainable=learn_words,
    #                     #input_length=2,
    #                     #weights=[np.array([[1,2],[3,4],[5,6]],'float32')]
    #                     weights=[model.syn0]
    #                     )
    #print word_vectors
    if word_vectors is None :
        kerasmodel.add_node(Embedding(vocab_size, vector_size,trainable=learn_words),name='embedword', input='iword')
    else:
        kerasmodel.add_node(Embedding(vocab_size, vector_size,trainable=learn_words,weights=[word_vectors]),name='embedword', input='iword')        
        
        # Embedding(vocab_size, vector_size,
        #           #weights=[model.syn0]
        #           ),
        # #embedword,
        # name='embedword', input='iword')
    if cbow_mean:
        kerasmodel.add_node(Lambda(lambda x:x.mean(1),output_shape=(vector_size,)),name='averageword',input='embedword')
    else:
        kerasmodel.add_node(Lambda(lambda x:x.sum(1),output_shape=(vector_size,)),name='averageword',input='embedword')

    # embedindex=Embedding(index_size, vector_size,
    #                      #trainable=learn_doctags,
    #                      #input_length=1,
    #                      #weights=[np.array([[10,20],[30,40]],'float32')]
    #                      #weights=[model.docvecs.doctag_syn0]
    #        )
    if doctag_vectors is None :
        kerasmodel.add_node(Embedding(index_size, vector_size,trainable=learn_doctags),name='embedindex', input='index')
    else:
        kerasmodel.add_node(Embedding(index_size, vector_size,trainable=learn_doctags,weights=[doctag_vectors]),name='embedindex', input='index')

        
    
        # #embedindex,
        # Embedding(index_size, vector_size),
        # name='embedindex', input='index')
    if cbow_mean:
        kerasmodel.add_node(Lambda(lambda x:x.mean(1),output_shape=(vector_size,)),name='averageindex',input='embedindex')
    else:
        kerasmodel.add_node(Lambda(lambda x:x.sum(1),output_shape=(vector_size,)),name='averageindex',input='embedindex')

    # ### This calcuation diffrent from original gensim. Because of this error https://github.com/fchollet/keras/issues/1474
    # kerasmodel.add_node(
    #     Merge([kerasmodel.nodes['averageword'],kerasmodel.nodes['averageindex']], mode='sum')
    #     #,output_shape=(vector_size,)
    #     ,name='average'
    #     #,name='sumembed'
    #     )
    #if model.hs:
    # hidden=Dense(code_dim, activation='sigmoid',
    #                  #b_constraint = keras.constraints.maxnorm(0),
    #                  #trainable=learn_hidden,
    #                  ,inputs=['averageindex','averageword'], merge_mode='sum'
    #                  #weights=[model.syn1.T,np.zeros((code_dim))
    #                  ]
    #                  )
    # else:
    #     hidden=Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),
    #                  #trainable=learn_hidden,
    #                  weights=[model.syn1neg.T,np.zeros((code_dim))]
    #                  )

    if hidden_vectors is None:
        kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),trainable=learn_hidden), name='sigmoid',inputs=['averageindex','averageword'], merge_mode='sum')
    else:
        kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),trainable=learn_hidden,weights=[hidden_vectors,np.zeros((code_dim))]), name='sigmoid',inputs=['averageindex','averageword'], merge_mode='sum') 
    # kerasmodel.add_node(
    #     #hidden,
    #     Dense(code_dim, activation='sigmoid',
    #                  #b_constraint = keras.constraints.maxnorm(0),
    #                  #trainable=learn_hidden,
                    
    #                  #weights=[model.syn1.T,np.zeros((code_dim))
    #                  ),
    #     name='sigmoid',
    #     #input='average'
    #     inputs=['averageindex','averageword'], merge_mode='sum'
    #     )


    kerasmodel.add_input(name='point', input_shape=(code_dim,), dtype=REAL)
    kerasmodel.add_node(kerasmodel.inputs['point'],name='pointnode')

    kerasmodel.add_output(name='code',inputs=['sigmoid','pointnode'], merge_mode='mul')

    kerasmodel.compile('rmsprop', {'code':'mse'})
    #print kerasmodel.predict({'index':np.array([[0],[1]]),'iword':np.array([[1,0],[1,1]]),'point':np.array([[1,1],[1,1]] )  })
    
    return kerasmodel
    
    
def build_keras_model_dm_concat(index_size,vector_size,vocab_size,code_dim,window_size,
                                learn_doctags=True, learn_words=True, learn_hidden=True,
                                model=None ,
                                word_vectors=None,doctag_vectors=None,hidden_vectors=None
                                ):

    kerasmodel = Graph()

    kerasmodel.add_input(name='iword' , input_shape=(1,), dtype=int)
    kerasmodel.add_input(name='index' , input_shape=(1,), dtype=int)


    # embedword=Embedding(vocab_size, vector_size,
    #                     input_length=2*window_size,
    #                     trainable=learn_words,
    #                     #weights=[np.array([[1,2],[3,4],[5,6]],'float32')]
    #                     weights=[model.syn0]
    #                     )
    kerasmodel.add_node(
        Embedding(vocab_size, vector_size,
                  input_length=2*window_size,
                  trainable=learn_words,
                  weights=[model.syn0]
                  #weights=[np.array([[1,2],[3,4],[5,6]],'float32')]
                  ),
        #embedword,
        name='embedword', input='iword')

    # embedindex=Embedding(index_size, vector_size,
    #                      input_length=1,
    #                      trainable=learn_doctags,
    #                      #weights=[np.array([[10,20],[30,40]],'float32')]
    #                      #weights=[model.docvecs.doctag_syn0]
    #                      )
    if doctag_vectors is None:
        kerasmodel.add_node(Embedding(index_size, vector_size,input_length=1,trainable=learn_doctags,), name='embedindex', input='index')
    else:
        kerasmodel.add_node(Embedding(index_size, vector_size,input_length=1,trainable=learn_doctags,weights=[doctag_vectors]), name='embedindex', input='index')

    kerasmodel.add_node(Flatten(),name='embedconcat',inputs=['embedindex','embedword'],merge_mode='concat', concat_axis=1)

    # if model.hs:
    #     hidden=Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),
    #                  trainable=learn_hidden,
    #                  weights=[model.syn1.T,np.zeros((code_dim))]
    #                  )
    # else:
    #     hidden=Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),
    #                  trainable=learn_hidden,
    #                  weights=[model.syn1neg.T,np.zeros((code_dim))]
    #                  )
        
    
    # hidden=Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),
    #                           trainable=learn_hidden
    #                           )
    
    kerasmodel.add_node(
        Dense(code_dim, activation='sigmoid', b_constraint = keras.constraints.maxnorm(0),
              #input_dim=(6,),
              trainable=learn_hidden,
              weights=[model.syn1.T,np.zeros((code_dim))]
              ),
        #hidden,
        name='sigmoid',
        input='embedconcat'
        #inputs=['flattenindex','flattenword'],merge_mode='concat'
        )

    kerasmodel.add_input(name='point', input_shape=(code_dim,), dtype=REAL)
    kerasmodel.add_node(kerasmodel.inputs['point'],name='pointnode')


    kerasmodel.add_output(name='code',inputs=['sigmoid','pointnode'], merge_mode='mul')

    kerasmodel.compile('rmsprop', {'code':'mse'})
    #print kerasmodel.predict({'index':np.array([[0],[1]]),'iword':np.array([[1,0],[1,1]]),'point':np.array([[1,1],[1,1]] )  }) 
    return kerasmodel
        
class Doc2VecKeras(gensim.models.doc2vec.Doc2Vec):

    def train(self, docs=None, batch_size=800,
              learn_doctags=True, learn_words=True, learn_hidden=True,iter=None,
              ):
        #print 'Doc2VecKeras.train'
        if iter!=None:
            self.iter=iter
        if docs==None:
            docs=self.docvecs
        
        vocab_size=len(self.vocab)
        index_size=len(self.docvecs)

        #batch_size=800 ##optimized 1G mem video card
        #batch_size=3200
        self.batch_size=batch_size
        batch_size=batch_size
        samples_per_epoch=int(self.window*2*sum(map(len,docs)))
        #print 'samples_per_epoch',samples_per_epoch
        
        #print 'Doc2VecKeras.train sg=',self.sg,learn_words,batch_size,samples_per_epoch/batch_size
        
        if self.hs and self.negative>0 :
            raise ValueError("both using hs and negative not implemented") 
        
        if self.sg:
            self.kerasmodel=build_keras_model_dbow(index_size=index_size,vector_size=self.vector_size,vocab_size=vocab_size,code_dim=vocab_size,
                                                   model=self,learn_doctags=learn_doctags, learn_words=learn_words, learn_hidden=learn_hidden)
   
            #dv0=copy.copy(self.kerasmodel.nodes['embedding'].get_weights()[0][0])
            
            # self.kerasmodel.fit_generator(
            #     train_batch_dbow(self, docs, self.alpha, batch_size=batch_size),
            #     samples_per_epoch=samples_per_epoch, nb_epoch=self.iter)
            count =0
            for g in train_batch_dbow(self, docs, self.alpha, batch_size=batch_size):
                self.kerasmodel.fit(g, nb_epoch=1, verbose=0)
                count +=1
                if count > self.iter * samples_per_epoch/batch_size :
                    break

            # print dv0
            # print self.kerasmodel.nodes['embedding'].get_weights()[0][0]
            # sys.exit()
            self.syn1=(self.kerasmodel.nodes['sigmoid'].get_weights()[0]).T


            self.docvecs.doctag_syn0=self.kerasmodel.nodes['embedding'].get_weights()[0]
            if self.hs:
                self.syn1=(self.kerasmodel.nodes['sigmoid'].get_weights()[0]).T
            if self.negative>0:
                self.syn1neg=(self.kerasmodel.nodes['sigmoid'].get_weights()[0]).T

            self.kerasmodel_infer=build_keras_model_dbow(index_size=1,vector_size=self.vector_size,vocab_size=vocab_size,code_dim=vocab_size,
                                                   model=self,learn_doctags=True, learn_words=False, learn_hidden=False
                                                   #hidden_vectors=self.syn1.T,
                                                         )

            
        else:
            if self.dm_concat:
                #print 'dm_concat'
                #print 'window',self.window
                self.kerasmodel=build_keras_model_dm_concat(index_size,self.vector_size,vocab_size,vocab_size,self.window,
                                                            model=self,
                                                            learn_doctags=learn_doctags, learn_words=learn_words, learn_hidden=learn_hidden,
                                                            doctag_vectors=self.docvecs.doctag_syn0
                                                            )

                # wv0=copy.copy(self.kerasmodel.nodes['embedword'].get_weights()[0][0])
                # dv0=copy.copy(self.kerasmodel.nodes['embedindex'].get_weights()[0][0])
                
                self.kerasmodel.fit_generator(
                    train_document_dm_concat(self, docs, batch_size=batch_size),
                    samples_per_epoch=samples_per_epoch, nb_epoch=self.iter, verbose=0)
                
                # count =0
                # for g in train_document_dm_concat(self, docs, batch_size=batch_size):
                #     self.kerasmodel.fit(g, nb_epoch=1, verbose=0)
                #     count +=1
                #     if count > self.iter * samples_per_epoch/batch_size :
                #         break

                # print wv0
                # print self.kerasmodel.nodes['embedword'].get_weights()[0][0]
                # print dv0
                # print self.kerasmodel.nodes['embedindex'].get_weights()[0][0]
                # sys.exit()
                
                self.docvecs.doctag_syn0=self.kerasmodel.nodes['embedindex'].get_weights()[0] 
                self.syn0=self.kerasmodel.nodes['embedword'].get_weights()[0]
                if self.hs:
                    self.syn1=(self.kerasmodel.nodes['sigmoid'].get_weights()[0]).T
                if self.negative>0:
                    self.syn1neg=(self.kerasmodel.nodes['sigmoid'].get_weights()[0]).T


                self.kerasmodel_infer=build_keras_model_dm_concat(1,self.vector_size,vocab_size,vocab_size,self.window,
                                                            model=self,
                                                            learn_doctags=True, learn_words=False, learn_hidden=False
                                                            )
                
            else:
                #print 'dm'
                self.kerasmodel=build_keras_model_dm(index_size,self.vector_size,vocab_size,vocab_size, model=self,
                                                     learn_doctags=learn_doctags, learn_words=learn_words, learn_hidden=learn_hidden,
                                                     word_vectors=self.syn0,
                                                     doctag_vectors=self.docvecs.doctag_syn0,
                                                     hidden_vectors=self.syn1.T,
                                                     cbow_mean=self.cbow_mean,
                                                     )

                #wv0=copy.copy(self.kerasmodel.nodes['embedword'].get_weights()[0][0])
                #dv0=copy.copy(self.kerasmodel.nodes['embedindex'].get_weights()[0][0])
                
                count =0
                # gn=train_batch_dm(self, docs, batch_size=batch_size)
                # count=0
                # for g in gn:
                #     print g
                #     count +=1
                #     if count >=2 :
                #         sys.exit()

                # print samples_per_epoch/batch_size
                # sys.exit()
                for g in train_batch_dm(self, docs, batch_size=batch_size):
                    self.kerasmodel.fit(g, nb_epoch=1, verbose=0)
                    count +=1
                    if count > self.iter * samples_per_epoch/batch_size :
                        break
                    
                # self.kerasmodel.fit_generator(
                #     train_batch_dm(self, docs, batch_size=batch_size),
                #     samples_per_epoch=samples_per_epoch, nb_epoch=self.iter)

                # print wv0
                # print self.kerasmodel.nodes['embedword'].get_weights()[0][0]
                # print dv0
                # print self.kerasmodel.nodes['embedindex'].get_weights()[0][0]
                # sys.exit()
                

                                
                self.docvecs.doctag_syn0=self.kerasmodel.nodes['embedindex'].get_weights()[0] 
                self.syn0=self.kerasmodel.nodes['embedword'].get_weights()[0]
                if self.hs:
                    self.syn1=(self.kerasmodel.nodes['sigmoid'].get_weights()[0]).T
                if self.negative>0:
                    self.syn1neg=(self.kerasmodel.nodes['sigmoid'].get_weights()[0]).T

                self.kerasmodel_infer=build_keras_model_dm(1,self.vector_size,vocab_size,vocab_size, model=self,
                                                           learn_doctags=True, learn_words=False, learn_hidden=False,
                                                           word_vectors=self.syn0,
                                                           #doctag_vectors=self.docvecs.doctag_syn0,
                                                           hidden_vectors=self.syn1.T,
                                                           cbow_mean=self.cbow_mean,
                                                           )
                

    def infer_vector_keras(self, doc_words, steps=10):
    #def infer_vector(self, doc_words, steps=15):        
        vocab_size=len(self.vocab)
        docs=LabeledListSentence([doc_words])
        batch_size=5
        #batch_size=10
        #batch_size=100
        #batch_size=1000
        samples_per_epoch=int(self.window*2*sum(map(len,docs)))
        
        count_max= steps * samples_per_epoch/batch_size +steps
        #print 'count_max',count_max
        # print self.kerasmodel_infer.nodes['embedword'].get_weights()
        # print self.kerasmodel_infer.nodes[ 'sigmoid'].get_weights()

        doctag_vectors = empty((1, self.vector_size), dtype=REAL)
        doctag_vectors[0] = self.seeded_vector(' '.join(doc_words))
        doctag_locks = ones(1, dtype=REAL)
        doctag_indexes = [0]
        #self.kerasmodel_infer.nodes.get_weights()
        self.kerasmodel_infer.nodes['embedindex'].set_weights([ doctag_vectors])
        count =0
        if self.sg:
            for g in train_batch_dbow(self, docs, self.alpha, batch_size=batch_size):
                self.kerasmodel_infer.fit(g, nb_epoch=1, verbose=0)
                count +=1
                if count > count_max:
                    break
        elif self.dm_concat:
            for g in train_document_dm_concat(self, docs, batch_size=batch_size):
                self.kerasmodel_infer.fit(g, nb_epoch=1, verbose=0)
                count +=1
                if count > count_max:
                    break
        else:
            for g in train_batch_dm(self, docs, batch_size=batch_size):
                self.kerasmodel_infer.fit(g, nb_epoch=1, verbose=0)
                count +=1
                if count > count_max:
                    break
        # print self.kerasmodel_infer.nodes['embedword'].get_weights()
        # print self.kerasmodel_infer.nodes[ 'sigmoid'].get_weights()
        vecs=self.kerasmodel_infer.nodes['embedindex'].get_weights()[0]
        return vecs[0]

                
    def train_with_word2vec_instance(self,docs,w2v,dm=None, **kwargs):
        if self.dm_concat and not w2v.null_word :
            raise ValueError("self.dm_concat=1 need Word2Vec(null_word=1)") #KeyError ?
        
        if dm == None :
            #if not self.dm_concat:
            self.sg = w2v.sg
        else:
            self.sg=(1+dm) % 2
            
        self.window = w2v.window 
        self.min_count =w2v.min_count
        self.sample =w2v.sample
        self.cbow_mean=w2v.cbow_mean
        
        self.negative = w2v.negative
        self.hs=w2v.hs
            
        #self.alpha = w2v.alpha 

        self.vector_size=w2v.vector_size
        if not self.dm_concat:
            self.layer1_size= w2v.layer1_size


        self.raw_vocab=w2v.raw_vocab
        self.index2word=w2v.index2word
        self.sorted_vocab = w2v.sorted_vocab

        self.vocab=w2v.vocab

        self.max_vocab_size = w2v.max_vocab_size

        
        #self.build_vocab(docs)
        for document_no, document in enumerate(docs):
            document_length = len(document.words)
            for tag in document.tags:
                self.docvecs.note_doctag(tag, document_no, document_length)
        self.reset_weights()


        self.syn0=w2v.syn0

        if w2v.hs:
            if not self.dm_concat:
                self.syn1=w2v.syn1
        if w2v.negative:
            if not self.dm_concat:
                self.syn1neg=w2v.syn1neg
            self.cum_table=w2v.cum_table

        
        self.train(docs,**kwargs)
        #self.train(docs,learn_words=learn_words,**kwargs)
        

class LabeledListSentence(object):
    def __init__(self, words_list):
        """
        words_list like:
        words_list = [
        ['human', 'interface', 'computer'],
        ['survey', 'user', 'computer', 'system', 'response', 'time'],
        ['eps', 'user', 'interface', 'system'],
        ]
        sentence = LabeledListSentence(words_list)
        """
        self.words_list = words_list
    
    def __getitem__(self, index):
        t = [t for t in self]
        return t[index]
    def __iter__(self):
        for i, words in enumerate(self.words_list):
            #yield LabeledSentence(words, ['SENT_{0}'.format(i)])
            yield gensim.models.doc2vec.TaggedDocument(words, [i])


def debug_fit():            
    kerasmodel=build_keras_model_dm(index_size=2,vector_size=2,vocab_size=3,code_dim=2,
                                    word_vectors=np.array([[1,2],[3,4],[5,6]]),
                                    doctag_vectors=np.array([[10,20],[30,40]]),
                                    hidden_vectors=np.array([[1,-1],[-1,1]]),
                                    learn_words=True
                                    )
    
    #kerasmodel=build_keras_model_dm_concat(index_size=2,vector_size=2,vocab_size=3,code_dim=2)
    print kerasmodel.predict({'index':np.array([[0],[1]]),'iword':np.array([[1,0],[1,1]]),'point':np.array([[1,1],[1,1]] )  })
    print 1.0/(1+math.exp(-(3+10-(4+20)) ))
    def mygen():
        while 1:
            bach_size=2
            # for i in range(2):
            #     x0=np.array([[i]], "int32")
            #     x1=np.array([[i]], "int32")
            #     x2=np.array([numpy.random.random((2))])
            #     y=np.array([numpy.random.random((2))])
            #print x1,x2,y
            #return 
            yield {'index': np.random.randint(2, size=(bach_size, 1)),
                       'iword': np.random.randint(3, size=(bach_size, 2)),
                       'point': np.random.random((bach_size,2)),
                       'code': np.random.random((bach_size,2))

            }
    g=mygen()
    print g.next()
    #sys.exit()
    
    #kerasmodel.fit_generator(mygen(),samples_per_epoch=100, nb_epoch=100)
    kerasmodel.fit({'index':np.array([[0],[1]]),'iword':np.array([[1,0],[1,1]]),'point':np.array([[1,1],[1,1]] ),'code':np.array([[1,1],[1,1]])  } )
    print kerasmodel.nodes['embedword'].get_weights()
    print kerasmodel.nodes['embedindex'].get_weights()
    kerasmodel.fit({'index':np.array([[0],[1]]),'iword':np.array([[1,0],[1,1]]),'point':np.array([[1,1],[1,1]] ),'code':np.array([[1,1],[1,1]])  } )
    print kerasmodel.nodes['embedword'].get_weights()
    print kerasmodel.nodes['embedindex'].get_weights()
    sys.exit()
    
if __name__ == "__main__":

    # debug_fit()
    # sys.exit()
    
    input_file = 'test.txt'
    doc1=gensim.models.doc2vec.TaggedLineDocument(input_file)
    for d in doc1:
        doc_words1=d.words
        break;
    d_size=5
    
    dvdbowk1=Doc2VecKeras(doc1,size= d_size,dm=0,iter=3)
    print dvdbowk1.docvecs[0]
    dvdbow1=gensim.models.doc2vec.Doc2Vec(doc1,size= d_size,dm=0)
    print dvdbow1.docvecs[0]

    #sys.exit()

    dvdmk1=Doc2VecKeras(doc1,size= d_size,dm=1,iter=10)
    print dvdmk1.docvecs[0]
    dvdm1=gensim.models.doc2vec.Doc2Vec(doc1,size= d_size,dm=1,iter=1)
    print dvdm1.docvecs[0]
    print dvdm1.infer_vector(doc_words1)
    print dvdmk1.infer_vector(doc_words1, steps=1)
    print dvdmk1.infer_vector(doc_words1, steps=100)
    #print dvdmk1.infer_vector_keras(doc_words1, steps=1000)
    print dvdmk1.infer_vector_keras(doc_words1)
    
    # sys.exit()

    dvdmck1=Doc2VecKeras(doc1,size= d_size,dm=1,dm_concat=1,iter=10)
    print dvdmck1.docvecs[0]
    dvdmc1=gensim.models.doc2vec.Doc2Vec(doc1,dm_concat=1,size= d_size,dm=1,iter=10)
    print dvdmc1.docvecs[0]
    print dvdmc1.infer_vector(doc_words1)
    print dvdmck1.infer_vector(doc_words1)
    print dvdmck1.infer_vector_keras(doc_words1)


    sys.exit()

    
    # dvdmk1=Doc2VecKeras(doc1,size=10,dm=1)
    # dvdm1=gensim.models.doc2vec.Doc2Vec(doc1,size=10,dm=1,iter=3)
    # print(dvdbow1.docvecs.most_similar(0))
    # print(dvdbowk1.docvecs.most_similar(0))
    # print(dvdmk1.docvecs.most_similar(0))
    # print(dvdm1.docvecs.most_similar(0))

    # dvdm1=gensim.models.doc2vec.Doc2Vec(doc1,dm=1,dm_concat=1,iter=3)
    # dvdmck1=Doc2VecKeras(doc1,dm=1,dm_concat=1,window=3)

    # # #dvdmk1=Doc2VecKeras(doc1,size=5,dm=1,dm_concat=1,window=3)
    # # dvdmck1=Doc2VecKeras(doc1,dm=1,dm_concat=1,iter=10)
    # # dvdmc1=gensim.models.doc2vec.Doc2Vec(doc1,dm=1,dm_concat=1,iter=3)
    # # print(dvdmck1.docvecs.most_similar(0))
    # # print(dvdmc1.docvecs.most_similar(0))


    from word2veckeras import Word2VecKeras
    
    # vk = Word2VecKeras(gensim.models.word2vec.LineSentence(input_file),null_word=1,iter=3)
    # dk=Doc2VecKeras()
    # dk.train_with_word2vec_instance(doc1,vk)

    #sys.exit()
    
    from nltk.corpus import brown 
    # print(brown.sents()[0])

    ### list should need
    brown_sents_sub=list(brown.sents()[:400])
    brown_docs_sub=LabeledListSentence(brown_sents_sub)

    # dvb=gensim.models.doc2vec.Doc2Vec(brown_docs_sub,dm=0)
    # print(dvb.docvecs.most_similar(0))
    # dvbk=Doc2VecKeras(brown_docs_sub,dm=0,iter=3)
    # print(dvbk.docvecs.most_similar(0))
    # # br = gensim.models.word2vec.Word2Vec(brown.sents())
    # # brk = Word2VecKeras(brown.sents(),iter=10)

    # dvb=gensim.models.doc2vec.Doc2Vec(brown_docs_sub,dm=1)
    # print(dvb.docvecs.most_similar(0))
    # dvbk=Doc2VecKeras(brown_docs_sub,dm=1,iter=3)
    # print(dvbk.docvecs.most_similar(0))


    # dvb=gensim.models.doc2vec.Doc2Vec(brown_docs_sub,dm=1,dm_concat=1)
    # print(dvb.docvecs.most_similar(0))
    # dvbk=Doc2VecKeras(brown_docs_sub,dm=1,dm_concat=1,iter=3)
    # print(dvbk.docvecs.most_similar(0))

    # #sys.exit()
    d_iter=10
    v_iter=10
    
    dvbdbow=gensim.models.doc2vec.Doc2Vec(brown_docs_sub,dm=0)
    dvbdbowk=Doc2VecKeras(brown_docs_sub,dm=0,iter=d_iter)
    dvbdm=gensim.models.doc2vec.Doc2Vec(brown_docs_sub,dm=1)
    dvbdmk=Doc2VecKeras(brown_docs_sub,dm=1,iter=d_iter)
    dvbdmc=gensim.models.doc2vec.Doc2Vec(brown_docs_sub,dm=1,dm_concat=1)
    dvbdmck=Doc2VecKeras(brown_docs_sub,dm=1,dm_concat=1,iter=d_iter)

    vbrs = gensim.models.word2vec.Word2Vec(brown_sents_sub,sg=1)
    vbrc = gensim.models.word2vec.Word2Vec(brown_sents_sub,sg=0)
    vbrsk = Word2VecKeras(brown_sents_sub,iter=v_iter,sg=1)
    vbrck = Word2VecKeras(brown_sents_sub,iter=v_iter,sg=0)
    dvvsdm=Doc2VecKeras()
    dvvsdm.train_with_word2vec_instance(brown_docs_sub,vbrs,dm=1,learn_words=False,iter=d_iter)
    dvvcdm=Doc2VecKeras()
    dvvcdm.train_with_word2vec_instance(brown_docs_sub,vbrc,dm=1,learn_words=False,iter=d_iter)
    dvvskdm=Doc2VecKeras()
    dvvskdm.train_with_word2vec_instance(brown_docs_sub,vbrsk,dm=1,learn_words=False,iter=d_iter)
    dvvckdm=Doc2VecKeras()
    dvvckdm.train_with_word2vec_instance(brown_docs_sub,vbrck,dm=1,learn_words=False,iter=d_iter)


    
    print(dvbdbow.docvecs.most_similar(0))
    print(dvbdbowk.docvecs.most_similar(0))    
    print(dvbdm.docvecs.most_similar(0))
    print(dvbdmk.docvecs.most_similar(0))
    print(dvbdmc.docvecs.most_similar(0))
    print(dvbdmck.docvecs.most_similar(0))

    
    print(dvvsdm.docvecs.most_similar(0))
    print(dvvskdm.docvecs.most_similar(0))
    print(dvvcdm.docvecs.most_similar(0))
    print(dvvckdm.docvecs.most_similar(0))

    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl.html


import logging
import sys
import os
from word2vec import Word2Vec, Sent2Vec, LineSentence, LineScoredSentence, ScoredSent2Vec
import nltk
import numpy
from random import shuffle


def mysentscore1(sent,scale):
    mywords=['can', 'could', 'may', 'might', 'must', 'will']
    return  [sent.count(w)*scale for w in mywords]

genres = ['news', 'religion', 'hobbies', 'science_fiction', 'romance', 'humor']

#brown_sents_genres=[[nltk.corpus.brown.sents(categories=[g]),g]  for g in genres]
#brown_sents_genres=[[nltk.corpus.brown.sents(categories=[g]),g]  for g in genres]
brown_sents_label_pre=[[ [sent,g] for sent in nltk.corpus.brown.sents(categories=[g]) ] for g in genres]

#print brown_sents_label_pre[0][0]

brown_sents_labels=[]
for n in range(len(genres)):
    brown_sents_labels.extend(brown_sents_label_pre[n])

#print brown_sents_label[1]
#sents=[sent_label[0] for sent_label in brown_sents_labels]
#print sents



def test_sentvec(sg_v,scale):

#sents_scores=[[s,mysentscore1(s)] for s in sents]

    sents_scores_labels=[[sl[0],mysentscore1(sl[0],scale),sl[1]] for sl  in brown_sents_labels]
    print len(sents_scores_labels)

    #print sents_scores_labels[0]
    #print sents_scores_labels[1]

    #sents_scores_labels2 = numpy.random.shuffle(sents_scores_labels)
    shuffle(sents_scores_labels)

    sents_scores=[[scl[0],scl[1]] for scl in sents_scores_labels]
    sents=[scl[0] for scl in sents_scores_labels]

    # print sents_scores_labels[0]
    # print sents_scores_labels[1]

    # print sents_scores[0]
    # print sents_scores[1]


    # sys.exit(0)




    #sg_v=0
    #sg_v=1

    modelbrw = Word2Vec(sents , size=100, window=5, sg=sg_v, min_count=5, workers=8)
    modelbrw.save('braword.model')
    modelbrw.save_word2vec_format('braword.vec')

    modelbrsc = ScoredSent2Vec(sents_scores, model_file='braword.model',sg=sg_v)
    modelbrsc.save_sent2vec_format('brasentsc.vec')

    modelbrs = Sent2Vec(sents, model_file='braword.model',sg=sg_v)
    modelbrs.save_sent2vec_format('brasent.vec')



    N=len(brown_sents_labels)
    #X=numpy.array([modelbrs.sents[n] for n in range(N)])
    X1 =[modelbrs.sents[n] for n in range(N)]
    X2 =[modelbrsc.sents[n] for n in range(N)]
    X3 =[numpy.append(modelbrs.sents[n],sents_scores[n][1])  for n in range(N)]
    #Y=[brown_sents_labels[n][1] for n in range(N)]
    Y=[scl[2] for scl in sents_scores_labels]
    #print Y2

    #print X2[0]
    #print X3[0]
    #print X[0]
    #print Y[1]



    from sklearn.linear_model import LogisticRegression
    from sklearn.cross_validation import cross_val_score

    # logreg = LogisticRegression() #C=1e5)
    # logreg.fit(X, Y)
    # print logreg.predict(X[0])




    scores= cross_val_score(LogisticRegression(), X1, Y, scoring='accuracy', cv=5)
    ret1=scores.mean()
    #sg=0 [ 0.53975265  0.51620507  0.57984679  0.52477876  0.53628319]
    #sg=1 [ 0.54240283  0.51738362  0.58603418  0.52330383  0.55103245]


    scores = cross_val_score(LogisticRegression(), X2, Y, scoring='accuracy', cv=5)
    ret2=scores.mean()
    #sg=0 [ 0.63309776  0.68031821  0.71243371  0.69852507  0.67758112]
    #sg=1 [ 0.64163722  0.69799646  0.72156747  0.70707965  0.69085546]

    scores = cross_val_score(LogisticRegression(), X3, Y, scoring='accuracy', cv=5)
    ret3=scores.mean()

    result=[sg_v,scale,ret1,ret2,ret3]
    print result

    return result


#print test_sentvec(0,0.1)

from sklearn.grid_search import ParameterSampler, ParameterGrid
#params={'sg_v':[0,1],'scale':[2.0**(-n) for n in range(20)]}
#params={'sg_v':[1],'scale':[2.0**(-n) for n in range(20)]}
params={'sg_v':[1],'scale':[2.0**(n+1) for n in range(20)]}
print params
param_list = list(ParameterGrid(params))
print param_list

result=[test_sentvec(**param) for param in param_list]
print result

    ## non shuffle
    # sg_v=0
    # 0.538902704141
    # 0.692475037989
    # 0.537310549242

    # sg_v=1 
    # 0.611053293365
    # 0.697653222928
    # 0.610875989755

    ## shuffled
    # sg_v=0 score 1/10
    # 0.426177333191
    # 0.444867961652
    # 0.431898204423

    # sg_v=1 score 1/100
    # 0.487316849967
    # 0.474215855992
    # 0.487140206225

    # sg_v=1 score 1/500
    # 0.479182138158
    # 0.470971988597
    # 0.479182138158



# [1, 1.0, 0.48743532980102361, 0.49350754275254866, 0.48495790558488017]
# 16964
# [1, 0.5, 0.47924301524751478, 0.48637491503720753, 0.47511290890913116]
# 16964
#[1, 0.25, 0.48218813817300277, 0.48684657689048044, 0.48083085848747642]
# [1, 2.0, 0.48372067147114794, 0.48861381114444152, 0.47994600770741264]
# 16964
# [1, 4.0, 0.48743206331836469, 0.49297451893533867, 0.48554398407042426]
# 16964
# [1, 8.0, 0.48189183232472538, 0.48802259092773193, 0.47929498966706136]
# 16964
# /home/niitsumalocal/src/python/gensim/scoredsentence2vec/word2vec.py:1292: RuntimeWarning: overflow encountered in exp
#   fac = 1.0 / (1.0 + exp(-dot(l1, l2c.T)))  #  propagate hidden -> output
# ^CTraceback (most recent call last):



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Licensed under the GNU Affero General Public License, version 3 - http://www.gnu.org/licenses/agpl-3.0.html

import sys
# is_py2 = sys.version[0] == '2'
# if is_py2:
#     import Queue as Queue
# else:
#     import queue as Queue
from Queue import Queue

from numpy import zeros, random, sum as np_sum, add as np_add, concatenate, \
    repeat as np_repeat, array, float32 as REAL, empty, ones, memmap as np_memmap, \
    sqrt, newaxis, ndarray, dot, vstack, dtype, divide as np_divide

import gensim.models.word2vec 

from six.moves import xrange, zip
from six import string_types, integer_types, itervalues


import random

import numpy as np
import copy

import keras.constraints

from keras.utils.np_utils import accuracy
from keras.models import Graph,Sequential
from keras.layers.core import Dense, Dropout, Activation, Merge, Flatten , Lambda
from keras.layers.embeddings import Embedding
from keras.optimizers import SGD
from keras.objectives import mse


#global_count=0

def queue_to_list(q,extract_size):
    """ Dump a Queue to a list 
    >>> q=Queue()
    >>> q.put(1)
    >>> q.put(2)
    >>> q.put(3)
    >>> l=queue_to_list(q,2)
    >>> l==[1,2] and q.qsize() ==1
    True
    """
    # A new list
    l = []
    count=0
    while q.qsize() > 0:
        count +=1
        if count >extract_size:
            break
        l.append(q.get())

    return l



def train_sg_pair(model, word, context_index, alpha, learn_vectors=True, learn_hidden=True,
                  context_vectors=None, context_locks=None,
                  scale=1.0
                  ):

    if word not in model.vocab:
        return
    predict_word = model.vocab[word]  # target word (NN output)
    
    if model.hs:
        y=np.zeros((len(model.vocab)), dtype=REAL)
        #y=np.zeros((len(model.vocab)), dtype='int8')
        x1code=np.zeros((len(model.vocab)), dtype='int8')
        x1code[predict_word.point]=1 #*scale
        #print np.linalg.norm(x1code-y),
        x1=predict_word.index
        y[predict_word.point]=predict_word.code
        x0=context_index
        #x1=predict_word.index
        return x0,x1,y

    if model.negative:
        x0=context_index
        y=np.zeros((len(model.vocab)), dtype='int8')
        x1=np.zeros((len(model.vocab)), dtype='int8')
        
        word_indices = [predict_word.index]
        while len(word_indices) < model.negative + 1:
            w = model.cum_table.searchsorted(model.random.randint(model.cum_table[-1]))
            if w != predict_word.index:
                word_indices.append(w)

        x1[word_indices]=1
        y[word_indices]=model.neg_labels
        return x0,x1,y ##missed in develop branch

def train_batch_sg(model, sentences, alpha, work=None,batch_size=100):
    
    batch_count=0

    train_x0=[[0]]*batch_size
    train_x1=[[0]]*batch_size
    train_y=[[0]]*batch_size

    while 1:
        #print( 'train_batch_sg')
        for i_sentence, sentence in enumerate(sentences):
            #print i_sentence,
            word_vocabs = [model.vocab[w] for w in sentence if w in model.vocab and
                           model.vocab[w].sample_int > model.random.rand() * 2**32]
            for pos, word in enumerate(word_vocabs):
                reduced_window = model.random.randint(model.window)  # `b` in the original word2vec code

                # now go over all words from the (reduced) window, predicting each one in turn
                start = max(0, pos - model.window + reduced_window)
                window_length=len(word_vocabs[start:(pos + model.window + 1 - reduced_window)])
                #print window_length,
                for pos2, word2 in enumerate(word_vocabs[start:(pos + model.window + 1 - reduced_window)], start):
                    # don't train on the `word` itself
                    if pos2 != pos:
                        xy=train_sg_pair(model, model.index2word[word.index], word2.index, alpha,scale=(window_length)/4)
                        if xy !=None:
                            (x0,x1,y)=xy
                            train_x0[batch_count]=[x0]
                            train_x1[batch_count]=[x1]
                            train_y[batch_count]=y
                            batch_count += 1
                            
                            if batch_count >= batch_size :
                                #yield { 'index':np.array(train_x0), 'point':np.array(train_x1), 'code':np.array(train_y)}
                                yield [np.array(train_x0), np.array(train_x1)], np.array(train_y)
                                batch_count=0


def generate_model_point_embed_array(model,index_size,code_dim):
    vocab_size=len(model.vocab)
    # index_size=vocab_size
    # code_dim=vocab_size
    #model_point_embed=np.zeros((index_size,code_dim), dtype='int8')
    model_point_embed=np.zeros((index_size,code_dim), dtype=REAL)
    for i in range(index_size):
        #print model.index2word[i],model.vocab[model.index2word[i]].point
        model_point_embed[i][model.vocab[model.index2word[i]].point]=1.0
    # print model_point_embed
    # sys.exit()
    return model_point_embed

def build_keras_model_sg(index_size,vector_size,vocab_size,code_dim,learn_vectors=True,learn_hidden=True,model=None):
    model_point_embed=generate_model_point_embed_array(model,index_size,code_dim)

    
    word = Sequential()
    word.add(Embedding(index_size,vector_size, input_length=1,weights=[model.syn0]))
    word.add(Flatten())
    word.add(Dense(output_dim=code_dim,
                   b_constraint = keras.constraints.maxnorm(0),
                   weights=[model.syn1.T, np.zeros((code_dim),'float32')]
                   ))

    point = Sequential()
    point.add(Embedding(index_size,code_dim, input_length=1,trainable=False,
                        weights=[
                            #[[0,1,1],[0,1,1],[0,1,1]]
                            model_point_embed
                                 ]))
    point.add(Flatten())

    kerasmodel= Sequential()
    kerasmodel.add(Merge([word, point], mode='mul'))

    rmsprop1=keras.optimizers.RMSprop(lr=0.01, rho=0.9, epsilon=1e-04)
    kerasmodel.compile(loss='mse', optimizer=rmsprop1)
    #kerasmodel.compile(loss='mse', optimizer='rmsprop')
    #kerasmodel.compile(loss='mse', optimizer='sgd')
    #kerasmodel.compile(loss='binary_crossentropy', optimizer='sgd')
    
    # kerasmodel = Graph()
    # kerasmodel.add_input(name='point', input_shape=(code_dim,), dtype=REAL)
    # kerasmodel.add_input(name='index' , input_shape=(1,), dtype=int)
    # kerasmodel.add_node(kerasmodel.inputs['point'],name='pointnode')
    # #kerasmodel.add_node(Embedding(index_size, vector_size, input_length=1,weights=[model.syn0]),name='embedding', input='index')
    # kerasmodel.add_node(Embedding(index_size, vector_size, input_length=1),name='embedding', input='index')
    # kerasmodel.add_node(Flatten(),name='embedflatten',input='embedding')
    # #kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),weights=[model.syn1.T,np.zeros((code_dim))]), name='sigmoid', input='embedflatten')
    # kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0)), name='sigmoid', input='embedflatten')
    # kerasmodel.add_output(name='code',inputs=['sigmoid','pointnode'], merge_mode='mul')
    # kerasmodel.compile('rmsprop', {'code':'mse'})
    
    return kerasmodel



def train_cbow_pair(model, word, input_word_indices, l=None, alpha=None, learn_vectors=True, learn_hidden=True):
    
    if model.hs:
        x0=input_word_indices
        #x1=np.zeros((len(model.vocab)), dtype=REAL)
        #x1[word.point]=1
        x1=[word.index]
        y=np.zeros((len(model.vocab)), dtype='int8')
        y[word.point]=word.code
        return x0,x1,y
    
    if model.negative:
        word_indices = [word.index]
        while len(word_indices) < model.negative + 1:
            w = model.cum_table.searchsorted(model.random.randint(model.cum_table[-1]))
            if w != word.index:
                word_indices.append(w)
        x0=input_word_indices
        #x1=np.zeros((len(model.vocab)), dtype=REAL)
        
        x1[word_indices]=1
        y=np.zeros((len(model.vocab)), dtype=REAL)
        y[word_indices]=model.neg_labels
        return x0,x1,y
        


def train_batch_cbow_xy_generator(model, sentences):
    while 1:
        for sentence in sentences:
            #print sentence
            word_vocabs = [model.vocab[w] for w in sentence if w in model.vocab and  model.vocab[w].sample_int > model.random.rand() * 2**32]
            for pos, word in enumerate(word_vocabs):
                reduced_window = model.random.randint(model.window)  # `b` in the original word2vec code
                start = max(0, pos - model.window + reduced_window)
                window_pos = enumerate(word_vocabs[start:(pos + model.window + 1 - reduced_window)], start)
                word2_indices = [word2.index for pos2, word2 in window_pos if (word2 is not None and pos2 != pos)]
                xy=train_cbow_pair(model, word , word2_indices , None, None)
                if xy !=None:
                    yield xy

def train_batch_cbow(model, sentences, alpha=None, work=None, neu1=None,batch_size=100):
    w_len_queue_dict={}
    w_len_queue=[]
    gen= train_batch_cbow_xy_generator(model, sentences)
    while 1:
        #print( 'train_batch_cbow')
        for xy in gen:
            if xy != None:
                w_len=len(xy[0])
                if w_len>0:
                    if w_len not in w_len_queue_dict:
                        w_len_queue_dict[w_len]=Queue()
                        w_len_queue.append(w_len)
                    w_len_queue_dict[w_len].put(xy)
            for w_len in w_len_queue:
                #print w_len,w_len_queue_dict[w_len]._qsize()
                if w_len_queue_dict[w_len].qsize() >= batch_size :
                    #print w_len_queue_dict[w_len]
                    l=queue_to_list(w_len_queue_dict[w_len],batch_size)
                    #train=zip(*l)
                    train=[[e[i] for e in l] for i in range(3)]
                    # yield { 'index':np.array(train[0]),
                    #         'point':np.array(train[1]),
                    #         'code':np.array(train[2])}
                    yield [np.array(train[0]),np.array(train[1])],np.array(train[2])

        
def build_keras_model_cbow(index_size,vector_size,vocab_size,code_dim,model=None,cbow_mean=False):
    
    model_point_embed=generate_model_point_embed_array(model,index_size,code_dim)
    #print model_point_embed
    word = Sequential()
    word.add(Embedding(index_size,vector_size,weights=[model.syn0]))
    word.add(Lambda(lambda x:x.sum(1),output_shape=(vector_size,)))
    word.add(Dense(output_dim=code_dim,
                   b_constraint = keras.constraints.maxnorm(0),
                   weights=[model.syn1.T, np.zeros((code_dim),'float32')]
                   ))
    point = Sequential()
    point.add(Embedding(index_size,code_dim, input_length=1,trainable=False,weights=[model_point_embed]))
    point.add(Flatten())

    kerasmodel= Sequential()
    kerasmodel.add(Merge([word, point], mode='mul'))

    kerasmodel.compile(loss='mse', optimizer='rmsprop')




    # kerasmodel = Graph()
    # kerasmodel.add_input(name='index' , input_shape=(1,), dtype=int)
    # kerasmodel.add_input(name='point', input_shape=(code_dim,), dtype=REAL)
    # kerasmodel.add_node(kerasmodel.inputs['point'],name='pointnode')

    # #kerasmodel.add_node(Embedding(index_size, vector_size,weights=[model.syn0]),name='embedding', input='index')
    # #kerasmodel.add_node(Embedding(index_size, vector_size,trainable=True,weights=[model.syn0]),name='embedding', input='index')
    # kerasmodel.add_node(Embedding(index_size, vector_size),name='embedding', input='index')
    # #kerasmodel.add_node(Embedding(index_size, vector_size,trainable=True),name='embedding', input='index')


    
    # if cbow_mean:
    #     kerasmodel.add_node(Lambda(lambda x:x.mean(1),output_shape=(vector_size,)),name='average',input='embedding')
    # else:
    #     kerasmodel.add_node(Lambda(lambda x:x.sum(1),output_shape=(vector_size,)),name='average',input='embedding')
    # #kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),weights=[model.syn1.T,np.zeros((code_dim))]), name='sigmoid', input='average')
    # #kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),trainable=True,weights=[model.syn1.T,np.zeros((code_dim))]), name='sigmoid', input='average')
    # kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0)), name='sigmoid', input='average')
    # #kerasmodel.add_node(Dense(code_dim, activation='sigmoid',b_constraint = keras.constraints.maxnorm(0),trainable=True), name='sigmoid', input='average')
    # kerasmodel.add_output(name='code',inputs=['sigmoid','pointnode'], merge_mode='mul')
    # kerasmodel.compile('rmsprop', {'code':'mse'})

    # # print dir(kerasmodel.nodes['embedding'])
    # # #print kerasmodel.nodes['embedding']
    # # #print kerasmodel.nodes['embedding'].params
    # # #print kerasmodel.nodes['embedding'].updates
    # # print kerasmodel.nodes['embedding'].get_config()
    # # #print kerasmodel.nodes['embedding'].get_params()
    # # sys.exit()

    
    return kerasmodel

                            

class Word2VecKeras(gensim.models.word2vec.Word2Vec):

     def train(self, sentences, total_words=None, word_count=0, batch_size=800, total_examples=None, queue_factor=2, report_delay=1):
        vocab_size=len(self.vocab)
        #print 'Word2VecKerastrain'
        #batch_size=800 ##optimized 1G mem video card
        #batch_size=800
        batch_size=batch_size
        #batch_size=3200
        #samples_per_epoch=int(self.window*2*sum(map(len,sentences)))
        #print 'samples_per_epoch',samples_per_epoch

        if self.hs and self.negative>0 :
            raise ValueError("both using hs and negative not implemented") 
        
        if self.sg:
            if not hasattr(self, 'kerasmodel') :
                self.kerasmodel=build_keras_model_sg(index_size=vocab_size,vector_size=self.vector_size,vocab_size=vocab_size,code_dim=vocab_size,model=self)

            #print self.kerasmodel.predict([np.array([[0],[1]], "int32"),np.array([[0],[1]], "int32")])
            #sys.exit()
            
            samples_per_epoch=int(self.window*2*sum(map(len,sentences)))
            batch_size=100*self.window/2
            #print samples_per_epoch,batch_size

            # samples_per_epoch=5000
            # #samples_per_epoch=9000
            #batch_size=3

            gen= train_batch_sg(self, sentences, self.alpha, work=None,batch_size=batch_size)
            
            # wv0=copy.copy(self.kerasmodel.nodes['embedding'].get_weights()[0][0])
            #wv0=copy.copy(self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0])

            #self.kerasmodel.fit_generator(train_batch_sg(self, sentences, self.alpha, work=None,batch_size=batch_size),samples_per_epoch=samples_per_epoch, nb_epoch=self.iter, verbose=0)
            self.kerasmodel.fit_generator(gen,samples_per_epoch=samples_per_epoch, nb_epoch=self.iter, verbose=0)
            
            #self.kerasmodel.fit_generator(gen,samples_per_epoch=samples_per_epoch, nb_epoch=self.iter,verbose=0)
            # count =0
            # for g in gen : #train_batch_sg(self, sentences, self.alpha, work=None,batch_size=batch_size):
            #     # print g[0]
            #     # print g[1]
            #     # print self.kerasmodel.predict(g[0])
            #     #wv0=copy.copy(self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0])
            #     self.kerasmodel.fit(g[0],g[1], nb_epoch=10, verbose=0)
            #     #print np.linalg.norm(wv0 - self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0]),
            #     #sys.exit()
            #     count +=1
            #     if count > self.iter * samples_per_epoch/batch_size :
            #         break

            
            # print wv0
            # # print self.kerasmodel.nodes['embedding'].get_weights()[0][0]
            # print self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0]
            # print np.linalg.norm(wv0 - self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0])
            # sys.exit()
            
            #self.syn0=self.kerasmodel.nodes['embedding'].get_weights()[0]
            self.syn0=self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0]
        else:
            if not hasattr(self, 'kerasmodel'):
                self.kerasmodel=build_keras_model_cbow(index_size=vocab_size,vector_size=self.vector_size,vocab_size=vocab_size,code_dim=vocab_size,model=self,cbow_mean=self.cbow_mean)

            # print self.kerasmodel.predict([np.array([[0],[1]], "int32"),np.array([[0],[1]], "int32")])
            # sys.exit()
                
            samples_per_epoch=int(sum(map(len,sentences)))
            batch_size=100

            # samples_per_epoch=5
            # batch_size=3
            
            # #samples_per_epoch=5000
            # samples_per_epoch=9000
            # batch_size=60
            
            # debug_print_index=3
            # wv0=copy.copy(self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0])
            # wv00=copy.copy(self.kerasmodel.nodes['embedding'].get_weights()[0][debug_print_index])
            # wv0=copy.copy(self.kerasmodel.nodes['embedding'].get_weights()[0])
            # ws10=copy.copy(self.kerasmodel.nodes['sigmoid'].get_weights()[0][debug_print_index][debug_print_index+1])
            # #ws1=copy.copy(self.kerasmodel.nodes['sigmoid'].get_weights()[0][debug_print_index])
            # ws1=copy.copy(self.kerasmodel.nodes['sigmoid'].get_weights()[0])
            
            gen=train_batch_cbow(self, sentences, self.alpha, work=None,batch_size=batch_size)

            self.kerasmodel.fit_generator(gen,samples_per_epoch=samples_per_epoch, nb_epoch=self.iter,verbose=0)

            # count =0
            # for g in gen : #train_batch_cbow(self, sentences, self.alpha, work=None,batch_size=batch_size):
            #     #print g[0]
            #     #print g[1]
            #     #print g['code'].shape,g['point'].shape,g['index']
            #     #print g['index']
            #     self.kerasmodel.fit(g[0],g[1], nb_epoch=1, verbose=0)
            #     #sys.exit()
            #     count +=1
            #     if count > self.iter * samples_per_epoch/batch_size :
            #         break

            # print wv00
            # print self.kerasmodel.nodes['embedding'].get_weights()[0][debug_print_index]
            # #print np.linalg.norm(wv0 - self.kerasmodel.nodes['embedding'].get_weights()[0][debug_print_index])
            # print np.linalg.norm(wv0 - self.kerasmodel.nodes['embedding'].get_weights()[0])
            # print ws10
            # print self.kerasmodel.nodes['sigmoid'].get_weights()[0][debug_print_index][debug_print_index+1]
            # #print self.kerasmodel.nodes['sigmoid'].get_weights()[0][debug_print_index]
            # #print np.linalg.norm(ws1 - self.kerasmodel.nodes['sigmoid'].get_weights()[0][debug_print_index])
            # print np.linalg.norm(ws1 - self.kerasmodel.nodes['sigmoid'].get_weights()[0])

            #print wv0
            # # print self.kerasmodel.nodes['embedding'].get_weights()[0][0]
            #print self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0]
            #print np.linalg.norm(wv0 - self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0])
            #sys.exit()
            
            
            #self.syn0=self.kerasmodel.nodes['embedding'].get_weights()[0]
            self.syn0=self.kerasmodel.layers[0].layers[0].layers[0].get_weights()[0]
            


if __name__ == "__main__":

    import doctest
    doctest.testmod()
    global_count=0

    from nltk.corpus import brown #, movie_reviews, treebank

    input_file = 'test.txt'
    
    v_iter=1
    sg_v=1
    d_size=10
    ns=[1,10,20,50,100]
    
    sents=gensim.models.word2vec.LineSentence(input_file)
    sents=list(brown.sents())[0:2000]
    #sents=list(brown.sents())
    
    # vsk = Word2VecKeras(gensim.models.word2vec.LineSentence(input_file),iter=v_iter)

    #sys.exit()
    # vs = gensim.models.word2vec.Word2Vec(gensim.models.word2vec.LineSentence(input_file))

    # print( vs.most_similar('the', topn=5))
    
    # vck = Word2VecKeras(gensim.models.word2vec.LineSentence(input_file),sg=0,iter=v_iter)
    # vc = gensim.models.word2vec.Word2Vec(gensim.models.word2vec.LineSentence(input_file),sg=0)
    # print( vck.most_similar('the', topn=5))
    # print( vc.most_similar('the', topn=5))
    

    vs1 = gensim.models.word2vec.Word2Vec(sents,size=d_size,sg=sg_v,iter=1)
    #print( vs1['the'])
    print( vs1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))
    vs1 = gensim.models.word2vec.Word2Vec(sents,size=d_size,sg=sg_v,iter=10)
    #print( vs1['the'])
    print( vs1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))
    
    vsk1 = Word2VecKeras(sents,size=d_size,sg=sg_v,iter=1)
    #print( vsk1['the'])
    print( vsk1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))
    vsk1 = Word2VecKeras(sents,size=d_size,sg=sg_v,iter=5)
    #print( vsk1['the'])
    print( vsk1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))
    #vsk1 = Word2VecKeras(sents,size=d_size,sg=sg_v,iter=v_iter)
    ##print( vsk1['the'])
    #print( vsk1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))
    vsk1.train(sents)
    #print( vsk1['the'])
    print( vsk1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))
    vsk1.train(sents)
    #print( vsk1['the'])
    print( vsk1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))

    for n in ns :
        print( n)
        vsk.iter=n
        vsk.train(brown_sents)
        print( vsk1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))

    
    # vck1 = Word2VecKeras(gensim.models.word2vec.LineSentence(input_file),sg=0,size=5,iter=v_iter)
    # vc1 = gensim.models.word2vec.Word2Vec(gensim.models.word2vec.LineSentence(input_file),sg=0,size=5,iter=3)
    # print vck1['the']
    # print vc1['the']

    sys.exit()

    ## negative sampling has bug
    
    # vsnk = Word2VecKeras(gensim.models.word2vec.LineSentence(input_file),iter=3,hs=0,negative=5)
    # vsn = gensim.models.word2vec.Word2Vec(gensim.models.word2vec.LineSentence(input_file),negative=5)
    # print( vsnk.most_similar('the', topn=5))
    # print( vsn.most_similar('the', topn=5))
    
    # vcnk = Word2VecKeras(gensim.models.word2vec.LineSentence(input_file),iter=3,sg=0,hs=0,negative=5)
    # vcn = gensim.models.word2vec.Word2Vec(gensim.models.word2vec.LineSentence(input_file),sg=0,negative=5)
    # print( vcnk.most_similar('the', topn=5))
    # print( vcn.most_similar('the', topn=5))


    #sys.exit()

    
    
    from nltk.corpus import brown #, movie_reviews, treebank
    #print(brown.sents()[0])
    #brown_sents=list(brown.sents())
    #brown_sents=brown.sents()
    #brown_sents=list(brown.sents()[:10000])
    brown_sents=list(brown.sents())[:2000]
    v_iter=1

    # brck = Word2VecKeras(brown_sents,iter=2,sg=0,size=4,window=3)
    # sys.exit()
    
    # br = gensim.models.word2vec.Word2Vec(brown_sents)
    # print br.most_similar_cosmul(positive=['woman', 'he'], negative=['man'], topn=10)
    # brk = Word2VecKeras(brown_sents,iter=v_iter)
    # #print( brk.most_similar('the', topn=5))
    # #print( br.most_similar('the', topn=5))
    # #print brk.most_similar_cosmul(positive=['france', 'england'], negative=['london'], topn=10)
    # #print br.most_similar_cosmul(positive=['france', 'england'], negative=['london'], topn=10)
    # #print brk.most_similar(positive=['woman', 'king'], negative=['man'])
    # #print br.most_similar(positive=['woman', 'king'], negative=['man'])
    # #print br.most_similar_cosmul(positive=['woman', 'husband'], negative=['man'], topn=10)
    # #print brk.most_similar_cosmul(positive=['woman', 'husband'], negative=['man'], topn=10)
    # print brk.most_similar_cosmul(positive=['woman', 'he'], negative=['man'], topn=10)
    
    

    # for x, y in model.most_similar(positive=["香川"], negative=["うどん"], topn=3):
    #     print x, y
    #sys.exit()
    ns=[1,10,20,50,100]
    #ns=[2]
    # ns=[200,400,1000]
    # for n in ns :
    #     print n
    #     brc = gensim.models.word2vec.Word2Vec(brown_sents,sg=0,iter=n)
    #     print brc.most_similar_cosmul(positive=['woman', 'he'], negative=['man'], topn=4)
    # sys.exit()
    brk1 = Word2VecKeras(brown_sents,size=40,iter=1,sg=sg_v)
    #print brk.most_similar_cosmul(positive=['woman', 'he'], negative=['man'], topn=4)
    print( brk1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))
    for n in ns :
        print( n)
        brk1.iter=n
        brk1.train(brown_sents)
        #print brk.most_similar_cosmul(positive=['woman', 'he'], negative=['man'], topn=4)
        print( brk1.most_similar_cosmul(positive=['she', 'him'], negative=['he'], topn=4))
    sys.exit()
        
    print( brc.most_similar('the', topn=5))
    print( brck.most_similar('the', topn=5))
    sys.exit()


    br1 = gensim.models.word2vec.Word2Vec(brown_sents,size=5,iter=v_iter)
    brk1 = Word2VecKeras(brown_sents,size=5,iter=3)
    print( brk1['the'])
    print( br1['the'])


    brc1 = gensim.models.word2vec.Word2Vec(brown_sents,sg=0,size=5,iter=v_iter)
    brck1 = Word2VecKeras(brown_sents,sg=0,size=5,iter=3)
    print( brck1['the'])
    print( brc1['the'])



    
    # brn = gensim.models.word2vec.Word2Vec(brown.sents(),negative=5)
    # brnk = Word2VecKeras(brown.sents(),iter=3,negative=5)
    # print( brnk.most_similar('the', topn=5))
    # print( brn.most_similar('the', topn=5))

    # brcn = gensim.models.word2vec.Word2Vec(brown.sents(),sg=0,negative=5)
    # brcnk = Word2VecKeras(brown.sents(),iter=3,sg=0,negative=5)
    # print( brcnk.most_similar('the', topn=5))
    # print( brcn.most_similar('the', topn=5))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl.html


"""

"""

import logging
import sys
import os
import numpy 
from word2vec import Word2Vec, Sent2Vec, LineSentence, LineScoredSentence, ScoredSent2Vec

logging.basicConfig(format='%(asctime)s : %(threadName)s : %(levelname)s : %(message)s', level=logging.INFO)
logging.info("running %s" % " ".join(sys.argv))

#sg_v=0
sg_v=1

input_file = 'test.txt'
modelw = Word2Vec(LineSentence(input_file), size=100, window=5, sg=sg_v, min_count=5, workers=8)
modelw.save(input_file + '.model')
modelw.save_word2vec_format(input_file + '.vec')

sent_file = 'sent.txt'
models = Sent2Vec(LineSentence(sent_file), model_file=input_file + '.model',sg=sg_v)
models.save_sent2vec_format(sent_file + '.vec')


sents=list(LineSentence(sent_file))

def mysentscore1(sent):
    #mywords=['can', 'could', 'may', 'might', 'must', 'will']
    mywords=['the', 'and', 'of','with']
    return  [sent.count(w)/10.0 for w in mywords] ##need 1/10.0 to avoid overflow

sents_scores=[[s,mysentscore1(s)] for s in sents]
#print sents_scores
modelsc = ScoredSent2Vec(sents_scores, model_file=input_file+ '.model',sg=sg_v)
modelsc.save_sent2vec_format(sent_file + 'sc.vec')


N=7
mylabel=['A','B','B','A','A','B','B']
Y=mylabel
print len(Y)
X1 =[models.sents[n] for n in range(N)]
X2 =[modelsc.sents[n] for n in range(N)]


from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import cross_val_score

scores= cross_val_score(LogisticRegression(), X1, Y, scoring='accuracy', cv=2)
print scores.mean()

scores= cross_val_score(LogisticRegression(), X2, Y, scoring='accuracy', cv=2)
print scores.mean()





program = os.path.basename(sys.argv[0])
logging.info("finished running %s" % program)
